﻿using System;
using System.IO;
using System.Windows.Forms;
using Microsoft.Win32;

namespace FileStuff
{
	// The fileOps class handles most of the file oriented routines and functions
	public class fileOps
	{
		// This is the path to the actual theme file being worked on.  It still requires the install dir
		public string themeFilePath = @"\Resource\Themes\Civ4\Civ4Theme_Common.thm";


		public fileOps()
		{
			// Default ctor
			
		}

		// Reads the entire theme file into a string array and returns it to the main program
		public string[ ] readInFile( string path )
		{
			
			// Make a copy of the path parameter
			string filePath = path;

			// The file is 520 lines long.  Read in extra to compensate for special things like UTF encoding
			string [] fileLines = new String[525];

			// If the file exixts, read from it.  Handle exception if not successful
			if ( filePath != null )
			{
				try
				{

					{
						fileLines = File.ReadAllLines( filePath );
					}

				}
				catch ( Exception ex )
				{
					MessageBox.Show( "Unable to read file.  The following exception has" +
						" occurred: " + ex.Message, "File Read Failed", MessageBoxButtons.OK );
				}
			}
			// Return the array
			return fileLines;
		
		}


		// The reverse of the read function
		public bool writeOutFile( string path, string[ ] fileLines )
		{
			bool success = false;

			string filePath = path;
			
			if ( filePath != null )
			{
				try
				{

					{
						File.WriteAllLines( filePath, fileLines );
						success = true;
					}

				}
				catch ( Exception ex )
				{
					MessageBox.Show( "Unable to save to file.  The following exception has" +
						" occurred: " + ex.Message, "File Save Failed", MessageBoxButtons.OK );
					
				}

			}

			// return a success code
			return success;
		}

		
		// Restores the bak file and makes a copy of the working file as a ".old" file that 
		// can be manually renamed if a mistake is made.
		public bool restoreFile( string path )
		{
			bool success = false;

			// Init some strings to hold file/path names
			string oldThmFile = String.Empty;
			string newThmFile = String.Empty;
			
			string filePath = locateFile(path);

			if ( File.Exists( filePath ) )
			{

				oldThmFile = Path.ChangeExtension( filePath, ".bak" );
				try
				{
					string query = "You are about to restore your original theme file \n" +
						"Do you wish to continue?";
					if ( MessageBox.Show( query, "Restore Theme", MessageBoxButtons.YesNo ) == DialogResult.Yes )
					{
						newThmFile = Path.ChangeExtension( filePath, ".old" );

						File.Copy( filePath, newThmFile, true );
						File.Copy( oldThmFile, filePath, true );
						MessageBox.Show( "Values restored to original state.", "File Copied", MessageBoxButtons.OK );
						

						success = true;
					}
				}
				catch ( Exception ex )
				{
					MessageBox.Show( "Unable to restore the original file.  The following exception has" +
						" occurred: " + ex.Message, "File Restore Failed", MessageBoxButtons.OK );
					
				}
			}

			else
			{
				success = true;


			}
				
			return success;
		}


		public string locateFile( string path )
		{
			// Make sure that the files can be found in order to restore them
			FileInfo fileFind = new FileInfo( path );
			if ( fileFind.Exists )
			{
				// If it exists, return the filepath
				return path;
			}

			else
			{
				return null;
			}

			
		}

		// Builds the complete path - separate method for future expansion
		public string locateFullPath( string regPath )
		{
			string installPath = regPath + themeFilePath;
			
			FileInfo themeFile = new FileInfo( installPath );
			if ( themeFile.Exists )
			{
				return installPath;
			}

			else
			{
				return null;
			}

		}

		
		// Gets the Civ install path from the registry
		public string getFilePathReg()
		{
			// This method checks the registry for the installation of Civ4 
			
			string regFilePath = String.Empty;

			// Registry key value constants for Civ 4
			string BTSREG_STANDARD = "Sid Meier's Civilization 4 - Beyond the Sword";
			string COMPLETE_REG = "Sid Meier's Civilization 4 Complete";
			string GOLD_REG = "Sid Meier's Civilization 4 Gold";

			// Open the first two keys in the hive, and then collect an array for the rest of the nodes
			RegistryKey hklm = Registry.LocalMachine;
			RegistryKey hkSoftware = hklm.OpenSubKey("Software");
			String [] nextKeys = hkSoftware.GetSubKeyNames();
			RegistryKey nextPath = hkSoftware.OpenSubKey( nextKeys[0] );
			
			// Loop through the keys
			foreach ( string s in nextKeys )
			{

				if ( s.Equals( "Firaxis Games" ) )
				{
					// If we're at the Firaxis keys, get the subkeys to find BTS, etc.
					RegistryKey firaxisPath = hkSoftware.OpenSubKey( s );
					String[ ] subKeys = firaxisPath.GetSubKeyNames();

					// Loop through the sub-keys
					foreach ( string k in subKeys )
					{
						// Check for all flavors of Civ4
						if ( ( k.Equals( BTSREG_STANDARD ) ) || ( k.Equals( COMPLETE_REG ) ) || ( k.Equals( GOLD_REG ) ) )
						{
							// If the Civ key is found, open it and get the install location string, then close the key
							RegistryKey hkgamePath = firaxisPath.OpenSubKey( k );
							regFilePath = hkgamePath.GetValue( "INSTALLDIR" ).ToString();
							hkgamePath.Close();

							// return the filePath to the calling method
							return regFilePath;

						}
					}
				}

				
			}

			// If it gets this far, it has failed, and will return an empty string.
			return regFilePath;
		}


		// End class
	}
}